import logging
import gnupg
import os

log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)
#handler = logging.FileHandler('gpg_handler.log')
handler2 = logging.StreamHandler()
formatter = logging.Formatter(
    '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
# handler.setFormatter(formatter)
handler2.setFormatter(formatter)
# log.addHandler(handler)
log.addHandler(handler2)


class Gpg:
    def __init__(self, gnupghome=''):
        self.gnupghome = gnupghome
        log.info('New session.\n')
        if len(gnupghome) == 0:
            log.debug("Path not specified. Setting default gnupg directory.")
            log.debug('Creating gnupg instance in {}'.format(
                "default directory."))
            self.gpg = gnupg.GPG()
            return None
        else:
            log.debug("Setting {} as gnupg directory".format(gnupghome))
            if os.path.exists(gnupghome):
                log.debug('Creating gnupg instance in {}'.format(gnupghome))
                self.gpg = gnupg.GPG(gnupghome=gnupghome)
                return None
            else:
                log.warning(
                    "path {} does not exist. Trying to create...".format(gnupghome))
                try:
                    log.info("Creating dir in {}".format(gnupghome))
                    os.mkdir(gnupghome)
                    log.debug('Creating gnupg instance in {}'.format(gnupghome))
                    self.gpg = gnupg.GPG(gnupghome=gnupghome)
                    return None
                except Exception as e:
                    log.error('Cant create dir {}'.format(e))
                    return None

    def list_keys(self, **kwargs):
        #TODO: multisearch
        try:
            keys = self.gpg.list_keys()
            if len(keys) == 0:
                log.warning("gpg database is empty.")
                return [], 'Empty database'
            set1 = set(keys[1].keys())
            set2 = set(kwargs.keys())
            if set2-set1 != set():
                log.warning('Wrong keyword {}'.format(set2-set1))
                return [], "Wrong keywords {}".format(set2-set1)
            elif kwargs == {}:
                log.debug('Returning all keys.')
                return keys, None
            else:
                results = []
                log.debug('Searching {} in keys.'.format(kwargs))
                for _dict in keys:
                    for keyword in kwargs:
                        if type(_dict[keyword]) == type(list()):
                            for value in _dict[keyword]:
                                if kwargs[keyword] in value:
                                    log.debug(
                                        'Match in {}.'.format(_dict[keyword]))
                                    if not _dict in results:
                                        results.append(_dict)
                        else:
                            if kwargs[keyword] in _dict[keyword]:
                                log.debug(
                                    'Match in {}.'.format(_dict[keyword]))
                                if not _dict in results:
                                    results.append(_dict)
                log.debug("Returning {} matches.".format(len(results)))
                return results, None
        except Exception as e:
            log.error("Error in list_keys(self)", e)
            return [], 'Error in Gpg.list_keys(self, **kwargs)'

    def encrypt(self, message, sign=None, file=None, **kwargs):
        # TODO FILE ENCRYPT
        try:
            log.debug("autosearch enabled.")
            recipients = []
            encrypted = []
            errors = ''
            log.debug("Searching recipients.")
            keys, error = self.list_keys(**kwargs)
            if error != None:
                log.error(error)
                return [], error
            for key in keys:
                if key['ownertrust'] == '-':
                    log.warning(
                        "Key {} is untrusted. Forcing...".format(key['uids']))
                _encrypted = self.gpg.encrypt(
                    str(message), key['keyid'], sign=sign, always_trust=True)
                encrypted.append(_encrypted.data.decode())
                if not _encrypted.ok:
                    errors += _encrypted.stderr + '\n'
            return encrypted, errors
        except Exception as e:
            log.error("Error in self.encrypt()", e)
            return [], "Error in self.encrypt()"

    def decrypt(self):
        pass

    def sign(self):
        pass

    def symmetric_encrypt(self):
        pass

    def symmetric_decrypt(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import logging
import getpass
import time
import ssl
import configparser as config
from xmppy.gpg_holder import Gpg
from optparse import OptionParser

import sleekxmpp

if sys.version_info < (3, 0):
    from sleekxmpp.util.misc_ops import setdefaultencoding
    setdefaultencoding('utf8')
else:
    raw_input = input


class Client(sleekxmpp.ClientXMPP):
    def __init__(self, jid, password, recipient, msg_function, reply_function, freq=120, msg_t=True, rpl_t=True, wait=False):
        if jid == "file":
            tmp = config.ConfigParser()
            check = tmp.read(password)
            if check == []:
                print("empty {}".format(password))
                exit()
            data = tmp["configfile"]
            if "jid" in data.keys() and "password" in data.keys():
                self.j = data["jid"]
                self.p = data["password"]
            else:
                print("Missing data in {}".format(password))
                exit()
            if recipient is None and msg_t:
                if "recipient" in data.keys():
                    self.recipient = data["recipient"]
                else:
                    print("Missing recipient in {}".format(password))
                    exit()
            else:
                self.recipient = recipient

        else:
            self.j = jid
            self.p = password
            self.recipient = recipient
        self.msg_function = msg_function
        self.reply_function = reply_function
        self.msg_t = msg_t
        self.rpl_t = rpl_t
        self.mm = []
        self.gpg_encryption = False
        self.ntime = freq  # seconds

        self.run(wait)

    def run(self, wait):
        if not wait:
            sleekxmpp.ClientXMPP.__init__(self, self.j, self.p)
            self.add_event_handler("session_start", self.start)
            if self.rpl_t:
                self.add_event_handler("message", self.message)
            self.register_plugin('xep_0030')  # Service Discovery
            self.register_plugin('xep_0004')  # Data Forms
            self.register_plugin('xep_0060')  # PubSub
            self.register_plugin('xep_0199')  # XMPP Ping
            self.ssl_version = ssl.PROTOCOL_TLSv1_2
            if self.connect():
                self.process(block=True)
                print("Done")
            else:
                print("Unable to connect.")

    @classmethod
    def receiveMessage(cls, jid, password, reply):
        logging.basicConfig(
            level=logging.ERROR, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        instance = cls(jid, password, None, None, reply,
                       freq=-1, msg_t=False, rpl_t=True)
        return instance

    @classmethod
    def sendMessage(cls, jid, password, recipient, message, freq=-1):
        logging.basicConfig(
            level=logging.ERROR, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        instance = cls(jid, password, recipient, message,
                       None, freq, msg_t=True, rpl_t=False)
        return instance

    @classmethod
    def initialize(cls, jid, password, recipient, message, reply, freq=10, wait=False):
        logging.basicConfig(
            level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        instance = cls(jid, password, recipient, message, reply,
                       freq=freq, msg_t=True, rpl_t=True, wait=wait)
        return instance

    def start(self, event):
        self.send_presence()
        self.get_roster()
        if self.msg_t:
            print("start() --msg_t->True")
            self._start_thread("chat_send", self.monitor)

    def monitor(self):
        while True:
            time.sleep(1)
            if int(time.time()) % self.ntime == 0 and self.ntime > 0:
                print("Sending...{}".format(self.MSG()))
                self.send_message(mto=self.recipient,
                                  mbody=self.MSG(), mtype='chat')
            elif self.ntime < 0:
                self.send_message(mto=self.recipient,
                                  mbody=self.MSG(), mtype='chat')
                self.disconnect(wait=True)
                break

    def message(self, msg):
        if msg['type'] in ('chat', 'normal'):
            msg.reply(self.RPL(msg["body"])).send()
            print(str(msg["body"]))
            self.mm = msg

    def enableGPG(self, uid, gpghome=''):
        self.gpg = Gpg(gpghome)
        self.uid = uid
        self.gpg_encryption = True
        return None

    def GPG(func):
        def wrap(self, *args, **kwargs):
            if self.gpg_encryption:
                message = self.gpg.encrypt(
                    func(self, *args, **kwargs), uids=self.uid)
            else:
                message = func(self, *args, **kwargs)
            return message
        return wrap

    @GPG
    def MSG(self):
        return self.msg_function()

    @GPG
    def RPL(self, msg):
        return self.reply_function(msg)


def main():
    optp = OptionParser()

    optp.add_option('-q', '--quiet', help='set logging to ERROR',
                    action='store_const', dest='loglevel',
                    const=logging.ERROR, default=logging.INFO)
    optp.add_option('-d', '--debug', help='set logging to DEBUG',
                    action='store_const', dest='loglevel',
                    const=logging.DEBUG, default=logging.INFO)
    optp.add_option('-v', '--verbose', help='set logging to COMM',
                    action='store_const', dest='loglevel',
                    const=5, default=logging.INFO)

# JID and password options.
    optp.add_option("-j", "--jid", dest="jid",
                    help="JID to use")
    optp.add_option("-p", "--password", dest="password",
                    help="password to use")
    optp.add_option("-t", "--to", dest="to",
                    help="JID to send the message to")
    optp.add_option("-m", "--message", dest="message",
                    help="message to send")

    opts, args = optp.parse_args()

    logging.basicConfig(level=opts.loglevel,
                        format='%(levelname)-8s %(message)s')

    if opts.jid is None:
        opts.jid = raw_input("Username: ")
    if opts.password is None:
        opts.password = getpass.getpass("Password: ")
    xmpp = Client.sendMessage(opts.jid, opts.password,
                           opts.to, lambda: opts.message)
